﻿package com.bezier{

	import flash.geom.Point;
	import flash.display.Sprite;
	import flash.display.MovieClip;
	import flash.display.Shape;
	import fl.motion.BezierSegment;

	public dynamic class BS_Bezier{
		
		private var point_1:Point;
		private var point_2:Point;
		private var handle_1:Point;
		private var handle_2:Point;
		private var container:Object;
		private var handles:Array;
		private var bezier:BezierSegment;
		private var markerContainer:Sprite;
		public var resolution:int;
		
		public function BS_Bezier(containerObj:Sprite, _resolution:uint):void{
			container=containerObj;
			markerContainer=new Sprite();
			markerContainer.x=0;
			markerContainer.y=0;
			MovieClip(container.parent).addChild(markerContainer);
			resolution=_resolution;
		}
		
		//The main-function for updating the display of our bezier:
		public function draw(handle1_x:Number, handle1_y:Number, handle2_x:Number, handle2_y:Number, point1_x:Number, point1_y:Number, point2_x:Number, point2_y:Number):void {			
			point_1=new Point(point1_x, point1_y);
			point_2=new Point(point2_x, point2_y);
			handle_1=new Point(handle1_x, handle1_y);
			handle_2=new Point(handle2_x, handle2_y);
			calculateBezier();
		}
		
		//for toggling the display of "joints"
		public function showJoints(){
			markerContainer.visible=true;
		}
		
		public function hideJoints(){
			markerContainer.visible=false;
		}
		
		//Changing resolution of our bezier. Drawing some markers while were at it:
		public function setResolution(res:uint):void{
			resolution=res;
			for(var j:uint=markerContainer.numChildren; j>0; j--){
				markerContainer.removeChild(markerContainer.getChildAt(j-1));
			}
			for (var i:uint=1; i<= resolution; i++) {
				this["joint_"+i]=new Shape();
				with(this["joint_"+i].graphics){
					beginFill(0x009900, 1);
					drawCircle(0, 0, 2);
					endFill();
				}
				markerContainer.addChild(this["joint_"+i]);
			}
			calculateBezier();
		}
		
		//Calculating the bezier:
		public function calculateBezier():void {
			//we need a new fl.motion BezierSegment. We will slice it into pieces, depending on the resolution.
			
			bezier=new BezierSegment(point_1, handle_1, handle_2, point_2);
			var t:Number=0; 										//t: the bezier-coordinates at a given time 
			var steps:Number=1/resolution;     						//how many slices do we need? We will increase "t" with this value as we move forward, step by step.
			
			var sliceStart:Point=point_1;							//the first point of our slice on the bezier-curve
			var sliceStartTangent:Array=[sliceStart,handle_1];   	//the control-Points of a  point on bezier at given time t 

			for (var i=1; i<= resolution; i++) {
				t+=steps;											//we move one step further
				var sliceEnd=bezier.getValue(t);
				var sliceEndTangent:Array=calcTangent(t);
				var quadControl:Point=checkIntersection(sliceStartTangent[0],sliceStartTangent[1],sliceEndTangent[0],sliceEndTangent[1]);
				
				//now we must check, wether distance between point-controlPoint is bigger than the distance point1-point2.
				//If so, we must divide the slice one more time in two, otherwise the plotted bezier-curve will contain errors.
				//Usually we should check this in a loop, until everything is o.k. I didn't care for now, therefore we get errors at low resolutions:
				
				if((Point.distance(quadControl, sliceStart)<=Point.distance(sliceStart, sliceEnd)) && (Point.distance(quadControl, sliceEnd)<=Point.distance(sliceStart, sliceEnd))){
					//everything o. k.
					sliceStartTangent=sliceEndTangent;
				}else{
					//not o. k. We must divide again.
					//go back one half of a step and calculate the middle:
					t-=steps/2;
					sliceEnd=bezier.getValue(t);
					sliceEndTangent=calcTangent(t);
					quadControl=checkIntersection(sliceStartTangent[0],sliceStartTangent[1],sliceEndTangent[0],sliceEndTangent[1]);
					with (container.graphics) {
						moveTo(sliceStart.x, sliceStart.y);
						curveTo(quadControl.x, quadControl.y, sliceEnd.x, sliceEnd.y);
					}
					//move forward and calculate the end of slice:
					t+=steps/2;
					sliceStart=sliceEnd;
					sliceStartTangent=sliceEndTangent;
					sliceEnd=bezier.getValue(t);
					sliceEndTangent=calcTangent(t);
					quadControl=checkIntersection(sliceStartTangent[0],sliceStartTangent[1],sliceEndTangent[0],sliceEndTangent[1]);
					sliceStartTangent=sliceEndTangent;
				}						
				
				with (container.graphics) {
					moveTo(sliceStart.x, sliceStart.y);
					curveTo(quadControl.x, quadControl.y, sliceEnd.x, sliceEnd.y);
				}
				
				//Draw some circles to mark the joints. Just for demonstration-purposes, can be removed in "serious" projects:
				try{
					markerContainer.getChildAt(i).x=sliceStart.x;
					markerContainer.getChildAt(i).y=sliceStart.y;
				}
				catch(e:RangeError){
				}
				//let's go on. The new start of the next slice is the end of the previous one:
				sliceStart=sliceEnd;			
			}
		}
		
		private function calcTangent(t:Number):Array {
			//decasteljau-algoritm fo finding the tangents at t:
			var m_1:Point=Point.interpolate(handle_1,point_1,t);
			var m_2:Point=Point.interpolate(handle_2,handle_1,t);
			var m_3:Point=Point.interpolate(point_2,handle_2,t);
			var h_3:Point=Point.interpolate(m_2,m_1,t);
			var h_4:Point=Point.interpolate(m_3,m_2,t);
			
			//returning the endpoints of handles. We will then use the intersection-point of the two lines (point-controlPoint)
			//as the control-point of our approximated quadratic bezier
			var handles:Array=new Array(h_3,h_4);
			return (handles);
		}
		
		private function checkIntersection(p1:Point, p2:Point, p3:Point, p4:Point):Point {
			//a function for checking intersectins of 2 lines. Line 1 is defined by p1 and p2, line 2 by p3 and p4
			var numerator_uA:Number=(p4.x-p3.x)*(p1.y-p3.y)-(p4.y-p3.y)*(p1.x-p3.x);
			var denominator_uA:Number=(p4.y-p3.y)*(p2.x-p1.x)-(p4.x-p3.x)*(p2.y-p1.y);

			var uA:Number=numerator_uA/denominator_uA;
			var numerator_uB:Number=(p2.x-p1.x)*(p1.y-p3.y)-(p2.y-p1.y)*(p1.x-p3.x);
			var denominator_uB:Number=denominator_uA;
			var uB:Number=numerator_uB/denominator_uB;
			
			var s1:Point=new Point();
				
			//do line-segments intersect?
			s1=(denominator_uA==0)?p1:new Point((p1.x+uA*(p2.x-p1.x)), (p1.y+uA*(p2.y-p1.y)));
			return (s1);
		}
		
	}// classs ende
}