﻿package com.gui{
	
	import flash.display.Sprite;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.display.Stage;
	import flash.display.Shape;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import com.bezier.GMA_Bezier;
	import com.bezier.BS_Bezier;
	import com.bezier.Handle;
	import com.events.UniversalEvent;
	import com.gui.palettes.slider.SliderComponent;
	import com.gui.palettes.ButtonComponent;	
	
	public class GUI{
		private var guiContainer:Sprite;
		private var coords:Array;
		private var GMAbezierContainer:Sprite;
		private var BSbezierContainer:Sprite;
		private var handleContainer:Sprite;
		private var bezierCol:int;
		private var mBezierCol:int;
		private var bezier:GMA_Bezier;
		private var motionBezier:BS_Bezier;
		private var handle1:Handle;
		private var handle2:Handle;
		private var theTimeline:Object;
		
		public function GUI(stageObj:Stage):void{
			bezierCol=0x003399;
			mBezierCol=0x006600;
			theTimeline=stageObj.getChildAt(0);
			initGui();
		}
		
		private function initGui():void{
			//some default-coordinates, so we get a nice bezier on startup: [control1.x, control1.y, control2.x, control2.y, startpoint.x, startpoint.y, endpoint.x, endpoint.y];
			coords=new Array(100, 60, 480, 70, 80, 300, 510, 380);
			
			//placing some container-sprites in the timeline:
			GMAbezierContainer= new Sprite();
			BSbezierContainer= new Sprite();
			var bg:Shape=new Shape();
			with (bg.graphics){
				beginFill(0xffffff, 0.5);
				drawRect(16,446, 270, 140);
				endFill;
			}
			
			//initializing the bezieres:
			theTimeline.addChild(GMAbezierContainer);
			theTimeline.addChild(BSbezierContainer);
			
			bezier=new GMA_Bezier(GMAbezierContainer);
			motionBezier=new BS_Bezier(BSbezierContainer, 10);
			handle1=new Handle([coords[0], coords[1], coords[4], coords[5]]);
			handle2=new Handle([coords[2], coords[3], coords[6], coords[7]]);

			GMAbezierContainer.graphics.lineStyle(1, bezierCol, 0.7);
			BSbezierContainer.graphics.lineStyle(1, mBezierCol, 0.7);
			bezier.draw(coords[0], coords[1], coords[2], coords[3], coords[4], coords[5], coords[6], coords[7]);
			motionBezier.draw(coords[0], coords[1], coords[2], coords[3], coords[4], coords[5], coords[6], coords[7]);
			
			//Placing GUI-Elemets:
			var label:String="Resolution";
			var slider:SliderComponent=new SliderComponent(label, motionBezier, "resolution", 110, 0, 1, 100, 15);
			slider.x=-24;
			slider.y=460;
			var showbezierBut:ButtonComponent=new ButtonComponent("Hide GMA-Bezier");
			var showMBezierBut:ButtonComponent=new ButtonComponent("Hide BS Bezier");
			var showJointsBut:ButtonComponent=new ButtonComponent("Hide Motion Bezier Joints");
			showbezierBut.x=showMBezierBut.x=showJointsBut.x=42;
			showbezierBut.y=490;
			showMBezierBut.y=520;
			showJointsBut.y=550;
			
			theTimeline.addChild(bg);
			theTimeline.addChild(slider);
			theTimeline.addChild(showbezierBut);
			theTimeline.addChild(showMBezierBut);
			theTimeline.addChild(showJointsBut);
			theTimeline.addChild(handle1);
			theTimeline.addChild(handle2);
			
			//adding some Listeners:
			handle1.addEventListener(UniversalEvent.CHANGE, updateCurve);
			handle2.addEventListener(UniversalEvent.CHANGE, updateCurve);
			slider.addEventListener(UniversalEvent.CHANGE, changeResolution);
			showbezierBut.addEventListener(MouseEvent.MOUSE_UP, toggleBezier);
			showMBezierBut.addEventListener(MouseEvent.MOUSE_UP, toggleMBezier);
			showJointsBut.addEventListener(MouseEvent.MOUSE_UP, toggleJoints);
			
		}
		
		private function toggleBezier(e){
			if(e.target.status==true){
				e.target.labeltxt="Show GMA-Bezier";
				e.target.status=false;
				GMAbezierContainer.visible=false;
			}else{
				e.target.labeltxt="Hide GMA-Bezier";
				e.target.status=true;
				GMAbezierContainer.visible=true;
			}
			e.target.changeTxt();
		}
		
		private function toggleMBezier(e){
			if(e.target.status==true){
				e.target.labeltxt="Show BS-Bezier";
				e.target.status=false;
				BSbezierContainer.visible=false;
			}else{
				e.target.labeltxt="Hide BS-Bezier";
				e.target.status=true;
				BSbezierContainer.visible=true;
			}
			e.target.changeTxt();
		}
		
		private function toggleJoints(e){
			if(e.target.status==true){
				e.target.labeltxt="Show BS-Bezier Joints";
				e.target.status=false;
				motionBezier.hideJoints();
			}else{
				e.target.labeltxt="Hide BS-Bezier Joints";
				e.target.status=true;
				motionBezier.showJoints();
			}
			e.target.changeTxt();
		}
		
		private function changeResolution(e:Object):void{
			BSbezierContainer.graphics.clear();
			BSbezierContainer.graphics.lineStyle(1, mBezierCol, 0.7);
			motionBezier.setResolution(e.arg[0]);
		}
		
		public function updateCurve(e:Event){
			coords=[handle1.coords[0], handle1.coords[1], handle2.coords[0], handle2.coords[1], handle1.coords[2], handle1.coords[3], handle2.coords[2], handle2.coords[3]];
			GMAbezierContainer.graphics.clear();
			GMAbezierContainer.graphics.lineStyle(1, bezierCol, 0.7);
			bezier.draw(coords[0], coords[1], coords[2], coords[3], coords[4], coords[5], coords[6], coords[7]);
	
			BSbezierContainer.graphics.clear();
			BSbezierContainer.graphics.lineStyle(1, mBezierCol, 0.7);
			motionBezier.draw(coords[0], coords[1], coords[2], coords[3], coords[4], coords[5], coords[6], coords[7]);	
		}
		
	}
}